import { IThing, IThingHittrSettings } from "./types";
/**
 * Automation for physics collisions and reactions.
 */
export declare class ThingHittr {
    /**
     * For each group name, the names of other groups it is allowed to hit.
     */
    private readonly groupHitLists;
    /**
     * Function generator for globalChecks.
     */
    private readonly globalCheckGenerator?;
    /**
     * Function generators for hitChecks.
     */
    private readonly hitCheckGenerators;
    /**
     * Function generators for HitCallbacks.
     */
    private readonly hitCallbackGenerators;
    /**
     * Check Functions for Things within groups to see if they're able to
     * collide in the first place.
     */
    private readonly generatedGlobalChecks;
    /**
     * Collision detection Functions to check two Things for collision.
     */
    private readonly generatedHitChecks;
    /**
     * Hit Function callbacks for when two Things do collide.
     */
    private readonly generatedHitCallbacks;
    /**
     * Hits checkers for when a Thing should have its hits detected.
     */
    private readonly generatedHitsChecks;
    /**
     * Initializes a new instance of the ThingHittr class.
     *
     * @param settings   Settings to be used for initialization.
     */
    constructor(settings?: IThingHittrSettings);
    /**
     * Caches global and hits checks for the given type if they do not yet exist
     * and have their generators defined
     *
     * @param groupName   What classification of Thing to cache hits for.
     * @param typeName   The specific Thing title to cache hits for.
     */
    cacheChecksForType(groupName: string, typeName: string): void;
    /**
     * Checks all hits for a Thing using its generated hits check.
     *
     * @param thing   The Thing to have hits checked.
     */
    checkHitsForThing(thing: IThing): void;
    /**
     * Checks whether two Things are hitting.
     *
     * @param thing   The primary Thing that may be hitting other.
     * @param other   The secondary Thing that may be being hit by thing.
     * @returns Whether the two Things are hitting.
     */
    checkHitForThings(thing: IThing, other: IThing): boolean;
    /**
     * Reacts to two Things hitting.
     *
     * @param thing   The primary Thing that is hitting other.
     * @param other   The secondary Thing that is being hit by thing.
     */
    runHitCallbackForThings(thing: IThing, other: IThing): void;
    /**
     * Function generator for a hits check for a specific Thing type.
     *
     * @param groupName   The type of the Things to generate for.
     * @returns A Function that can check all hits for a Thing of the given type.
     */
    private generateHitsCheck;
    /**
     * Runs the Function in the group that maps to the two Things' types. If it doesn't
     * yet exist, it is created.
     *
     * @param group   The group of Functions to use.
     * @param thing   The primary Thing reacting to other.
     * @param other   The secondary Thing that thing is reacting to.
     * @returns The result of the ThingFunction from the group.
     */
    private runThingsFunctionSafely;
    /**
     * Generates the list of group names each group is allowed to hit.
     *
     * @param group   A summary of group containers.
     */
    private generateGroupHitLists;
}
//# sourceMappingURL=ThingHittr.d.ts.map