define(["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.ThingHittr = void 0;
    /**
     * Automation for physics collisions and reactions.
     */
    class ThingHittr {
        /**
         * Initializes a new instance of the ThingHittr class.
         *
         * @param settings   Settings to be used for initialization.
         */
        constructor(settings = {}) {
            this.globalCheckGenerator = settings.globalCheckGenerator;
            this.hitCheckGenerators = settings.hitCheckGenerators || {};
            this.hitCallbackGenerators = settings.hitCallbackGenerators || {};
            this.generatedHitChecks = {};
            this.generatedHitCallbacks = {};
            this.generatedGlobalChecks = {};
            this.generatedHitsChecks = {};
            this.groupHitLists = this.generateGroupHitLists(this.hitCheckGenerators);
        }
        /**
         * Caches global and hits checks for the given type if they do not yet exist
         * and have their generators defined
         *
         * @param groupName   What classification of Thing to cache hits for.
         * @param typeName   The specific Thing title to cache hits for.
         */
        cacheChecksForType(groupName, typeName) {
            if (!{}.hasOwnProperty.call(this.generatedGlobalChecks, groupName) &&
                this.globalCheckGenerator) {
                this.generatedGlobalChecks[typeName] = this.globalCheckGenerator();
            }
            if (!{}.hasOwnProperty.call(this.generatedHitsChecks, groupName) &&
                {}.hasOwnProperty.call(this.hitCheckGenerators, groupName)) {
                this.generatedHitsChecks[typeName] = this.generateHitsCheck(groupName);
            }
        }
        /**
         * Checks all hits for a Thing using its generated hits check.
         *
         * @param thing   The Thing to have hits checked.
         */
        checkHitsForThing(thing) {
            this.generatedHitsChecks[thing.title](thing);
        }
        /**
         * Checks whether two Things are hitting.
         *
         * @param thing   The primary Thing that may be hitting other.
         * @param other   The secondary Thing that may be being hit by thing.
         * @returns Whether the two Things are hitting.
         */
        checkHitForThings(thing, other) {
            return !!this.runThingsFunctionSafely(this.generatedHitChecks, thing, other, this.hitCheckGenerators);
        }
        /**
         * Reacts to two Things hitting.
         *
         * @param thing   The primary Thing that is hitting other.
         * @param other   The secondary Thing that is being hit by thing.
         */
        runHitCallbackForThings(thing, other) {
            this.runThingsFunctionSafely(this.generatedHitCallbacks, thing, other, this.hitCallbackGenerators);
        }
        /**
         * Function generator for a hits check for a specific Thing type.
         *
         * @param groupName   The type of the Things to generate for.
         * @returns A Function that can check all hits for a Thing of the given type.
         */
        generateHitsCheck(groupName) {
            /**
             * Collision detection Function for a Thing. For each Quadrant the Thing
             * is in, for all groups within that Function that the Thing's group is
             * allowed to collide with, it is checked for collision with the Things
             * in that group. For each Thing it does collide with, the appropriate
             * hit Function is called.
             *
             * @param thing   A Thing to check collision detection for.
             */
            return (thing) => {
                // Don't do anything if the thing shouldn't be checking
                if ({}.hasOwnProperty.call(this.generatedGlobalChecks, groupName) &&
                    !this.generatedGlobalChecks[groupName](thing)) {
                    return;
                }
                // For each quadrant the Thing is in...
                for (let i = 0; i < thing.numQuadrants; i += 1) {
                    // For each group within that quadrant the Thing may collide with...
                    for (const groupName of this.groupHitLists[thing.groupType]) {
                        // For each other Thing in the group that should be checked...
                        for (let j = 0; j < thing.quadrants[i].numthings[groupName]; j += 1) {
                            const other = thing.quadrants[i].things[groupName][j];
                            // If they are the same, breaking to prevent double hits
                            if (thing === other) {
                                break;
                            }
                            // Do nothing if other can't collide in the first place
                            if (!this.generatedGlobalChecks[other.title](other)) {
                                continue;
                            }
                            // If they do hit, call the corresponding hitCallback
                            if (this.checkHitForThings(thing, other)) {
                                this.runHitCallbackForThings(thing, other);
                            }
                        }
                    }
                }
            };
        }
        /**
         * Runs the Function in the group that maps to the two Things' types. If it doesn't
         * yet exist, it is created.
         *
         * @param group   The group of Functions to use.
         * @param thing   The primary Thing reacting to other.
         * @param other   The secondary Thing that thing is reacting to.
         * @returns The result of the ThingFunction from the group.
         */
        runThingsFunctionSafely(group, thing, other, generators) {
            const typeThing = thing.title;
            const typeOther = other.title;
            let container = group[typeThing];
            if (container === undefined) {
                container = group[typeThing] = {};
            }
            let check = container[typeOther];
            if (check === undefined) {
                check = container[typeOther] = generators[thing.groupType][other.groupType]();
            }
            return check(thing, other);
        }
        /**
         * Generates the list of group names each group is allowed to hit.
         *
         * @param group   A summary of group containers.
         */
        generateGroupHitLists(group) {
            const output = {};
            for (const i in group) {
                if ({}.hasOwnProperty.call(group, i)) {
                    output[i] = Object.keys(group[i]);
                }
            }
            return output;
        }
    }
    exports.ThingHittr = ThingHittr;
});
//# sourceMappingURL=ThingHittr.js.map